classdef Bookie
    %BOOKIE is basically a "simple simulator" in the sence that it can
    %predict results based on an inferred probability distribution based on
    %the odds. It is different from a simulator-object (and
    %competitior-object  -- which also can calculate probabilities directly 
    %from a database) in that it will arrange bets for you as well. 
    
    properties
        setup
        results
        odds
    end
    
    methods
        function obj = Bookie(varargin)
            % This ammounts to reading the ODDS database, picking out the
            % correct bookie, and populating the results-structure.
            
            [ league, season, bookie ] = process_options(...
                varargin, ...
                'league', 'PL', ...
                'season', '2011-12', ...
                'bookie', 'WilliamHill');
            obj.setup.league = league;
            obj.setup.season = season;
            obj.setup.bookie = bookie;
            obj.odds = load(['../DATA/' season '/' league '-odds.mat']);
            %eval(['obj.odds = obj.odds.' bookie ';']);
            obj= setfield(obj, 'odds',  getfield(obj.odds, bookie));
            
        end
        function [newBankroll, totalBets, totalPayback, ...
                calculatedProbs, offeredOdds, actualResults] = ArrangeBets(obj, punter, db, varargin)
            % What we do here is to arrange a "shootout" between the obj -
            % bookie and a punter. The punter MUST offer a PredictGame
            % method, but appart from that can be whatever (so, can be a
            % simulator OR a bookie).
            % bankRoll is the available amount of money when the betting
            % starts.
            % Strategy is a char saying which money management strategy to
            % use. A number of them is on offer:
            % - Kelly
            % - FixedAmount
            % - FixedReturn
            % - VarianceAdjusted
			% - MarkovitzValue
            % Additionally, there is an number sent over, which has its own
            % interpretation, depending on which strategy we use. If it is
            % Kelly, we give the minimum expected gain (odds*probability)
            % to use. Default is 1.05.
            % For FixedBet, it is that fixed amount.
            % For FixedReturn, it is the fixed return (meaning we will bet
            % strategyNumber/odds for a given game)
            % db + varargin are used to generate the list of games.
            % For VarianceAdjusted, it is the scaler of which we multiply
            % to get the thing bet "reasonable" amounts (not only in the
            % range 0.001 - 0.3, which we often see otherwise)
            % For Markovitz, the number sent in needs to be between 0 and
            % 1, where 0 means the target return of the combined bet is the
            % average of all potential bets, while value equals 1 means we
            % will only accept bets with highest payback.
            % Once this is set, the algorithm chooses a set of bets to make
            % sure the return of the "portfolio" is as required, while at
            % the same time minimizing the risk (variance of return).
            %
            % Note! Since e.g. Kelly has this thing about being always
            % betting depending on the current bankroll, I have decided to
            % let the bankRoll at the beginning of the method decide the
            % bets for all bets in that set of bets. (Otherwise the somewhat
            % arbitrary ordering returned by GameList would be of major
            % importance...) A potential problem is that the sum of all
            % bets is larger than the bankroll (This can also happen for
            % other bet strategies). To avoid this problem, I sum up all
            % bets to be wagered, and normalize if the sum is too large.
            %
            % Norman Fenton reported a "not-long-shot bias", in one of his
            % papers, claiming that bookmakers often overrated the
            % favorites and made the long-shots less-than-appropriately
            % valued, because punters prefers the longshots (better
            % winnings whenever they do win). To work against this effect,
            % we have the 'onlyFavourites' option, which refuses to bet on
            % anything but the most probable result.
            additionalInputs = varargin;
            if length(additionalInputs)==1, additionalInputs=additionalInputs{1}; end
            [ verbose, onlyFavourites, bank, strategy, kellyMinimum, ...
                fixedBetValue, VarianceAdjustedValue, fixedReturnValue, MarkovitzValue, ~] = process_options(...
                additionalInputs, ...
                'verbose', 1, ...
                'onlyFavourites', 0, ...
                'initialBankroll', 1000, ...
                'bettingStrategy', 'FixedReturn', ...
                'KellyMinimum', 1, ...
                'FixedBetValue', 1, ...
                'VarianceAdjustedValue', 10, ...
                'FixedReturnValue', 1, ...
                'MarkovitzValue', .5);
            
            
            % gList holds the list of games and the true results
            quietArguments = replace_options(varargin, {'verbose', 0}, ...
                {'onlyFavourites', 'initialBankroll', 'BettingStrategy', 'KellyMinimum', ...
                'FixedBetValue', 'VarianceAdjustedValue', 'FixedReturnValue', 'MarkovitzValue' });
            gList = GameList(db, quietArguments);
            % punterProbabilities holds the list of games and the assumed
            % probability distributions
            punterProbabilities = punter.PredictGame(db, quietArguments);
            punts = zeros(gList.GetNoGames(), 1);		% How much we bet for each game
            payBackOdds = zeros(gList.GetNoGames(), 1); % This one is zero if loosing bet, offered odds otherwise
            betOption  = zeros(gList.GetNoGames(), 1);  % This is the best option (1==Home, 20=Draw, 3==Away)
			offeredOdds  = zeros(gList.GetNoGames(), 3); % Odds on offer for each of the outcomes
            calculatedProbs = zeros(gList.GetNoGames(), 3); % Probabilities calculated for each of the outcomes
            actualResults = zeros(gList.GetNoGames(), 3); % The actual outcome of the game
            
            for gameIdx = 1 : gList.GetNoGames()
                relevantOdds  = (obj.GetOdds(gList.GetGame(gameIdx)))';
				offeredOdds(gameIdx, :) = relevantOdds';
                probs = punterProbabilities.GetGame(gameIdx).winner;
				calculatedProbs (gameIdx, :) = probs';
                actualResults(gameIdx, :) = gList.GetGame(gameIdx).winner';
                if onlyFavourites
                    % Refuse to put money on anything but the favourite.
                    % Implemented by allocating a zero probability to the
                    % other outcomes, and then those outcomes will never be
                    % worthy of putting money on, obviously.
                    mostProbResult = max(probs);
                    probs(probs < mostProbResult) = 0;
                end
                
                [gain, betOption(gameIdx)] = max(relevantOdds .* probs);
                chosenPunt = 0;
                
                switch lower(strategy)
                    case 'kelly'
                        if gain > kellyMinimum
                            chosenPunt = bank * (probs(betOption(gameIdx)) * relevantOdds(betOption(gameIdx)) -1) / (relevantOdds(betOption(gameIdx)) - 1);
                        end
                    case 'fixedbet'
                        if gain > 1, chosenPunt = fixedBetValue; end
                    case 'fixedreturn'
                        if gain > 1, chosenPunt = fixedReturnValue ./ relevantOdds(betOption(gameIdx)); end
                    case 'varianceadjusted'
                        % The task here is to maximize expected return
                        % minus variance of return. I got this from the
                        % paper by Rue and Salvesen
                        if gain > 1, chosenPunt = min(bank, VarianceAdjustedValue ./ (2*relevantOdds(betOption(gameIdx))*(1-probs(betOption(gameIdx))))); end 
					case 'markovitz'
                        % Markovitz analyzes all bets at the same time.
                        % Wait until we are done with the game list
                        chosenPunt = 1;
                    otherwise
                        warning('Illegal strategy. Bets will be 0 all the way');
                end
                
                payBackOdds(gameIdx) = relevantOdds(betOption(gameIdx)) * gList.GetGame(gameIdx).winner(betOption(gameIdx));
                punts(gameIdx) = chosenPunt;
            end
            
			if strcmpi(strategy, 'Markovitz')
                noGames = gList.GetNoGames();
                returnsMatrix = calculatedProbs .* offeredOdds;
                [R, I] = max(returnsMatrix, [], 2);
                Chooser = zeros(size(returnsMatrix));
                Chooser(sub2ind(size(returnsMatrix), 1:length(I), I')) = 1;
                Sigma = diag(sum(Chooser.* calculatedProbs .* (1-calculatedProbs) .* offeredOdds, 2));
                targetValue = (1-MarkovitzValue)*mean(R) + MarkovitzValue * max(R);
                
                warning off;
                [punts, ~, flag] = quadprog( 2*Sigma, zeros(noGames, 1), [], [], ...
                    [R';ones(1,noGames)], [targetValue;1], zeros(noGames,1), ones(noGames, 1), [], optimset('Display','off'));
                warning on;
                if flag ~= 1, punts = zeros(noGames,1); end
            end
			
			punts = punts ./ max(1, sum(punts) / bank); % Scale -- punts will not be larger than bankroll
            newBankroll = bank + punts' *  payBackOdds - sum(punts);
            
            if verbose
                
                if verbose > 1
                    %Report back on all the fun
                    outcomes = {'HOME', 'DRAW', 'AWAY'};
                    for gameIdx = 1 : gList.GetNoGames()
                        relevantOdds  = (obj.GetOdds(gList.GetGame(gameIdx)))';
                        probs = punterProbabilities.GetGame(gameIdx).winner;
                        gList.GetGame(gameIdx).PresentGame();
                        fprintf('   Probabilities (odds) --- Home: %5.3f (%3.1f), Draw: %5.3f (%3.1f), Away: %5.3f (%3.1f)\n',...
                            probs(1), relevantOdds(1), probs(2), relevantOdds(2), probs(3), relevantOdds(3));
                        if punts(gameIdx) > 0
                            fprintf('   Betting strategy %s gives bet %4.2f (of bankroll %4.1f) on "%s". Winnings: %5.2f \n',...
                                strategy, punts(gameIdx), bank, char(outcomes{betOption(gameIdx)}), punts(gameIdx)*payBackOdds(gameIdx));
                        else
                            fprintf('   NOT PLAYABLE\n');
                        end
                    end
                end
                if (gList.GetNoGames() > 1) || (verbose==1)
                    fprintf('\n*********************************************************************\n');
                    fprintf('******  BETTING SUMMARY  ********************************************\n');
                    fprintf('*********************************************************************\n');
                    fprintf('\tResults for punter: %s \n', punter.GetName());
                    fprintf('\tBetting against: %s \n', obj.GetName());
                    fprintf('\tStrategy:  %s\n', strategy);
                    fprintf('\tBankroll at start:\t%7.2f\n\tTotal bets: \t\t%7.2f\n\tTotal payback:\t\t%7.2f\n\tNew bankroll: \t\t%7.2f\n',...
                        bank, sum(punts),  punts' * (payBackOdds), newBankroll );
                    fprintf('\tPunter gain pct: \t%7.2f%%\n\tAvg. bookie gain pct:\t%7.2f%%\n', ...
                        100*(punts' *  payBackOdds - sum(punts))/(1E-6+sum(punts)), ...
                        100*(obj.CalculateValue('verbose', 0)-1) );
                    fprintf('*********************************************************************\n');
                end
                
            end
            % Prepare return-objects.
            totalBets = sum(punts);
            totalPayback = punts' *  payBackOdds;
        end
        
        
        function [bankRoll,accumulatedBetsArray, accumulatedReturns, betsArray, returnsArray] = SeasonSimulation(obj, db, varargin)
            % What we do here is to arrange a "shootout" between the obj -
            % bookie and a punter. The punter MUST offer methods
            % - GenerateSamples
            % - PredictGame
            % initialBank is the available amount of money when the betting
            % starts.
            % Strategy is a char saying which money management strategy to
            % use. A number of them is on offer:
            % - Kelly
            % - FixedBet
            % - FixedReturn
            % - VarianceAdjusted
			% - Markovitz
            % Additionally, the value of strategyNumber has its own
            % interpretation, depending on which strategy we use. If it is
            % Kelly, we give the minimum expected gain (odds*probability)
            % to use. 
            % For FixedBet, it is that fixed amount.
            % For FixedReturn, it is the fixed return (meaning we will bet
            % strategyNumber/odds for a given game)
            % db + varargin are used to generate the list of games, and
            % rund the simulator
            %
            % The simulator is evaluated using obj.ArrangeBets, so check
            % that one for further settings
            [ startRound, verbose, initialBank, usesDetailedInfo, ~] = process_options(...
                varargin, ...
                'startRound', 1 + floor(db.GetNoRounds()/2), ...
                'verbose', 1, ...
                'initialBankroll', 1000, ...
                'usesDetailedInfo', 0);
            
            inputArguments = replace_options(varargin,{}, {'startRound'});
            simulatorArguments = replace_options(inputArguments,{'verbose', max(0, verbose-2)}, ...
                {'startRound', 'BettingStrategy', 'Round', 'FixedBetValue', 'KellyMinimum', ...
                'FixedReturnValue', 'VarianceAdjustedValue', 'MarkovitzValue', 'InitialBankroll', 'onlyFavourites'});
            bettingArguments = replace_options(inputArguments,{'verbose', max(0, verbose-1)},...
                {'JAGSfile', 'nchains','nburnin','nsamples', 'nthinning', 'doparallel'});
            punter = Simulator();
            bankRoll = initialBank;
			theProbs = []; theResults = [];
            accumulatedBets = 0;
            origDB = db;
            accumulatedBetsArray = zeros(db.GetNoRounds(),1);
            accumulatedReturns = zeros(db.GetNoRounds(),1);
            betsArray = zeros(db.GetNoRounds(),1);
            returnsArray = zeros(db.GetNoRounds(),1);
            totalBet = 0;
            totalPayback = 0;
            for myRound = startRound:db.GetNoRounds()
                bettingArguments = replace_options(bettingArguments, {'round', myRound, 'InitialBankRoll', bankRoll});
                if ~usesDetailedInfo
                    db = origDB.HideResult( 'round', myRound : db.GetNoRounds() );
                else
                    db = origDB.HideResult_DetailedDataVersion('round', myRound:db.GetNoRounds());
                end
                tic;
                punter = punter.GenerateSamples( db, simulatorArguments );
                toc;
                [bankRoll, bets, paybacks, calculatedProbs, ~, actualResults] = obj.ArrangeBets(punter, db, bettingArguments);
                accumulatedBets = accumulatedBets + sum(bets);
				theProbs = [theProbs; calculatedProbs(:)];
                theResults = [theResults; actualResults(:)];
				
				totalBet = totalBet+bets;
                totalPayback = totalPayback+paybacks;
                accumulatedBetsArray(myRound)=totalBet;
                accumulatedReturns(myRound) = totalPayback;
                
                betsArray(myRound)=bets;
                returnsArray(myRound)=paybacks;
                
				if verbose
                    fprintf('Round %2d: Bets: %7.2f, Returns: %7.2f, Bankroll: %7.2f\n', ...
                        myRound, bets, paybacks, bankRoll);
                    fprintf('Round %2d: Total spent on bets so far:\t%7.2f\nTotal Payback so far: \t\t%7.2f\n',...
                        myRound,totalBet, totalPayback);
                end
            end
            
            if verbose
                fprintf('\n*********************************************************************\n');
                fprintf('******  BETTING SUMMARY  ********************************************\n');
                fprintf('*********************************************************************\n');
                fprintf('\tResults for punter: %s \n', punter.GetName());
                fprintf('\tBetting against: %s \n', obj.GetName());
                fprintf('\tBankroll at start:\t%7.2f\n\tNew bankroll: \t\t%7.2f\n',...
                    initialBank, bankRoll);
				%% New Helge print
				fprintf('\tPunter gain pct: \t%7.2f%%\n\tAvg. bookie gain pct:\t%7.2f%%\n', ...
                    100*(bankRoll-initialBank)/(1E-6+accumulatedBets), ...
                    100*(obj.CalculateValue('verbose', 0)-1) );
				%% New Martin Print
				fprintf('\tTotal spent on bets:\t%7.2f\nTotal Payback: \t\t%7.2f\n',...
                    totalBet, totalPayback);
					
                fprintf('*********************************************************************\n');
				
				
				figure();                
                [theProbs, idx]=sort(theProbs);
                theResults = theResults(idx);
                plot(theProbs, theResults, 'b.'); hold on;
                xlim([0 1]);
                
                % Approximation: The probability is a smoothed
                % average (a k-NN type regression) of the 0/1
                % results of each outcome. Since the number of
                % points can be very small, this is combined with a
                % Gaussian(.5,1) in a Bayes-type update
                
                xx = linspace(0, 1, 1000);
                bandWidth = .025;
                priorWeight = 2*normpdf(0,0,bandWidth);
                
                for idx = 1:length(xx)
                    w = normpdf(theProbs, xx(idx), bandWidth );
                    sumW = sum(w)+eps('single');
                    probs(idx) =        sumW/(sumW+priorWeight) * (w' * theResults/sumW) + ...
                        priorWeight/(sumW+priorWeight) * .5;
                    sd(idx) =          (sumW/(sumW+priorWeight))^2 * (1-probs(idx))*probs(idx) * sum(w .^2) / (sum(w))^2  + ...
                        (priorWeight/(sumW+priorWeight))^2 * 1;
                    sd(idx) = sqrt(sd(idx));
                    
                end
                % Shadow-area for the +- 2* stdev; thick line for mean
                lowerBound = max(0, probs - 2*sd);
                upperBound = min(1, probs + 2*sd);
                bound = [lowerBound flipdim(upperBound, 2)];
                %xVals = [theOdds' flipdim(theOdds', 2)];
                xVals = [xx flipdim(xx, 2)];
                fill(xVals, bound, [8 5 5]/8, 'EdgeColor', [8 5 5]/8, 'FaceAlpha', .5);
                
                plot(xx, probs, 'r-', 'LineWidth', 2);
                plot(xx, xx, 'g--', 'LineWidth', 2);
                hold off;
			
			end
            
            
        end
        
        function name = GetName(obj)
            name = char(obj.setup.bookie);
        end
        function value = CalculateValue(obj, varargin)
            [ verbose] = process_options(...
                varargin, ...
                'verbose', 0);
            
            value = 1./obj.odds(:, 3:end);
            value = sum(value,2);
            value = mean(1./value);
            
            if verbose
                fprintf('Value of %s for %s in the season %s is %6.4f\n', ...
                    char(obj.setup.bookie), char(obj.setup.league), char(obj.setup.season), value);
            end
        end
		
		function OK = Verify(obj, varargin)
            %payBackPct = obj.CalculateValue('verbose', 0);
            [ verbose] = process_options(...
                varargin, ...
                'verbose', 1);
            
            
            db = Database();
            db = db.LoadDataset('league', obj.setup.league,'season', obj.setup.season);
            gl = GameList(db, 'round', 1:db.GetNoRounds());
            
            theResults = zeros(db.GetNoGames, 3);
            theOdds = theResults;
            
            observedGames = 0;
            for gmIdx = 1:gl.GetNoGames
                gm = gl.GetGame(gmIdx);
                if gm.gameSimulated == 0
                    observedGames = observedGames  + 1;
                    theResults(observedGames,:) = gm.winner;
                    theOdds(observedGames,:) = obj.GetOdds(gm);
                end
            end
            theOdds(observedGames+1:end,:)=[];
            theResults(observedGames+1:end,:)=[];
            
            theOdds=theOdds(:);
            theResults=theResults(:);
            [theOdds, idx]=sort(theOdds);
            theResults = theResults(idx);
            
            if verbose
                figure();
                
                
                semilogx(theOdds, theResults, 'b.'); hold on;
                xlim([min(theOdds) 10]);
                xl = xlim();
                
                
                % Approximation: The probability is a smoothed
                % average (a k-NN type regression) of the 0/1
                % results of each outcome. Since the number of
                % points can be very small, this is combined with a
                % Gaussian(.5,1) in a Bayes-type update
                
                xx = (exp(linspace(log(xl(1)), log(xl(end)), 1000)));
                bandWidth = .025;
                priorWeight = 2*normpdf(0,0,bandWidth);
                
                probs = zeros(size(xx));
                sd = ones(size(xx));
                
                for idx = 1:length(xx)
                    w = normpdf(theOdds, xx(idx), bandWidth );
                    sumW = sum(w)+eps('single');
                    probs(idx) =        sumW/(sumW+priorWeight) * (w' * theResults/sumW) + ...
                        priorWeight/(sumW+priorWeight) * .5;
                    sd(idx) =          (sumW/(sumW+priorWeight))^2 * (1-probs(idx))*probs(idx) * sum(w .^2) / (sum(w))^2  + ...
                        (priorWeight/(sumW+priorWeight))^2 * 1;
                    sd(idx) = sqrt(sd(idx));
                    
                end
                % Shadow-area for the +- 2* stdev; thick line for mean
                lowerBound = max(0, probs - 2*sd);
                upperBound = min(1, probs + 2*sd);
                bound = [lowerBound flipdim(upperBound, 2)];
                %xVals = [theOdds' flipdim(theOdds', 2)];
                xVals = [xx flipdim(xx, 2)];
                fill(xVals, bound, [8 5 5]/8, 'EdgeColor', [8 5 5]/8, 'FaceAlpha', .5);
                
                plot(xx, probs, 'r-', 'LineWidth', 2);
                plot(xx, 1 ./ xx, 'g--', 'LineWidth', 2);
                hold off;
            end
            OK=1;
        end
		
		
        function relevantOdds  = GetOdds(obj, aGame)
            chooser = (obj.odds(:,1)== aGame.homeTeamIdx) & ...
                (obj.odds(:,2)== aGame.awayTeamIdx);
            % -- will be a vector of zeros, with a single '1'
            if sum(chooser) ~=1
                warning('Not able to find odds for this game:');
                aGame.PresentGame();
                return;
            end
            relevantOdds = obj.odds(chooser, 3:5);
        end
        function result = PredictGame(obj, db, varargin)
            additionalInput = varargin;
            if length(additionalInput)==1, additionalInput=additionalInput{1}; end;
            result = GameList(db, additionalInput);
            for theGame = 1:result.GetNoGames()
                % Predict game in the list
                % 1. Pick out odds
                relevantOdds = obj.GetOdds(result.GetGame(theGame));
                % 2. Identitify that goals scored are not kept/guessed
                result.list{theGame}.homeGoals = nan;
                result.list{theGame}.awayGoals = nan;
                % 3. Save the distribution over result
                result.list{theGame}.winner = ((1./relevantOdds) / sum(1./relevantOdds))';
                % 4. Let distribution over scored goals show void
                result.list{theGame}.goalDistributionHome = [];
                result.list{theGame}.goalDistributionAway = [];
                % 5. Remember that this game is simulated.
                result.list{theGame}.gameSimulated = 1;
            end
        end
    end
end

