classdef Database
    %DATABASE Makes all the interfacing to the data; loading, masking,
    %releasing, whatever
    
    properties
        % The setup of the league
        noTeams  % no teams in the league
        noRounds  % no Rounds in total (given as 2*(noTeams-1), obviously
        teamNames % An array of chars
        playsAtHome % If playsAtHome (i, t) = 1, then team i plays at home in round t
        opposition % if opposition(i, t) = j, then team i plays team j in round t.
        gameDate   % gameDate(i, t) gives the "date" when team <i> played the 
                   % game set up for it in round <t>. Value is an integer,
                   % giving the no. days since start of season.
        gameOrder  % If gameOrder(i, t) = j, it means that the t'th game of the season
                   % played by team i was the one initially scheduled as
                   % round j. Mostly, gameOrder(i, t) = t, but postponing a
                   % single game will mess this up.
        
        % The results-part
        goalsScored
        firedShots
        %%%%%%%%%%%%%%%%%% NEW
        shotsOnTarget
        shotsBlocked
        possession
        firedPasses
        passesOnTarget
        wonContests
        tackles
        airials
        %%%%%%%%%%%%%%%%%%
        % Book-keeping part
        season % What season are we looking at
        league % What LEAGUE are we looking at
    end
    
    properties (Access = private)
        % These are the real (un-hidden) information. Useful to have, e.g.,
        % when I try to evaluate a method, and want to compare predictions
        % (based on the observations in the results part of the public
        % results-info with what actually happened.
        actualGoalsScored
        actualFiredShots
    end
    
    methods (Access = private)
        function obj = UpdateAfterNoTeamChange(obj,value)
            obj.noRounds = max(0, 2*(value-1));
            obj.teamNames = cell(1,value);
            obj.playsAtHome = zeros(obj.noTeams, obj.noRounds);
            obj.opposition = zeros(obj.noTeams, obj.noRounds);
            obj.gameDate = zeros(obj.noTeams, obj.noRounds);
            obj.gameOrder = zeros(obj.noTeams, obj.noRounds);
            obj.goalsScored = zeros(obj.noTeams, obj.noRounds);
            obj.firedShots = zeros(obj.noTeams, obj.noRounds);
            %%%%%%%%% NEW
            obj.shotsOnTarget = zeros(obj.noTeams, obj.noRounds);
            obj.shotsBlocked = zeros(obj.noTeams, obj.noRounds);
            obj.possession = zeros(obj.noTeams, obj.noRounds);
            obj.firedPasses = zeros(obj.noTeams, obj.noRounds);
            obj.passesOnTarget = zeros(obj.noTeams, obj.noRounds);
            obj.wonContests = zeros(obj.noTeams, obj.noRounds);
            obj.tackles = zeros(obj.noTeams, obj.noRounds);
            obj.airials = zeros(obj.noTeams, obj.noRounds);
            %%%%%%%%%
        end
    end
    
    methods
        % Creation and reading from file
        function obj = Database()
            obj.noTeams = 0;
        end
        function obj = LoadDataset(obj, varargin)
            [ myLeague, mySeason ] = process_options(...
                varargin, ...
                'league', 'PL', ...
                'season', '2011-12');
            
            sourceData = load(['../DATA/' mySeason '/' myLeague '.mat']);
			obj.league = myLeague;
			obj.season = mySeason;
            obj.noTeams = length(sourceData.teamNames);
            obj.teamNames = sourceData.teamNames;
            
            
            [obj.opposition, obj.playsAtHome,obj.gameDate, obj.gameOrder, obj.goalsScored, obj.firedShots...
                obj.shotsOnTarget, obj.shotsBlocked, obj.possession, obj.firedPasses, obj.passesOnTarget, ...
                obj.wonContests, obj.tackles, obj.airials] =...
                obj.TranslateGame2TeamRound(sourceData.schedule, sourceData.results);
            
%             
%             for match = 1:size(sourceData.schedule)
%                 obj.opposition(sourceData.schedule(match,2), sourceData.schedule(match,1)) = sourceData.schedule(match,3);
%                 obj.opposition(sourceData.schedule(match,3), sourceData.schedule(match,1)) = sourceData.schedule(match,2);
%                 obj.playsAtHome(sourceData.schedule(match,2), sourceData.schedule(match,1)) = 1;
%                 
%             end
%             % And finally recode the results-matrix. The matrix has 6 columns,
%             % holding he following information
%             % 1. Home - team
%             % 2. Away - team
%             % 3. HomeTeam goals
%             % 4. AwayTeam goals
%             % 5. HomeTeam shots
%             % 6. AwayTeam shots
%             %
%             % The reason for not just moving the information as above, when
%             % info about who played when, is that we don't know if all games
%             % have been played as and when they should --> Must "search"
%             % correct index for this to work.
%             
%             for match = 1:size(sourceData.results, 1)
%                 homeTeam = sourceData.results(match,1);
%                 awayTeam = sourceData.results(match,2);
%                 
%                 % Look for which game this is
%                 gameIdx = find(obj.opposition(homeTeam,:)==awayTeam);
%                 % gameIdx has two indexes, as the two teams have played
%                 % eachother twice.
%                 % Chose the one where <homeTeam> has played at home...
%                 if obj.playsAtHome(homeTeam, gameIdx(1))
%                     gameIdx = gameIdx(1);
%                 else
%                     gameIdx = gameIdx(2);
%                 end
%                 % And we are ready:
%                 obj.goalsScored(homeTeam, gameIdx) = sourceData.results(match, 3);
%                 obj.goalsScored(awayTeam, gameIdx) = sourceData.results(match, 4);
%                 obj.firedShots(homeTeam, gameIdx)  = sourceData.results(match, 5);
%                 obj.firedShots(awayTeam, gameIdx)  = sourceData.results(match, 6);
%             end
            
            % Put info into the private properties/data-containers
            obj.actualFiredShots = obj.firedShots;
            obj.actualGoalsScored= obj.goalsScored;
            
        end
        
        % Utility stuff
        function count = CountGames(obj, varargin)
            % Count the number of times "something" happens.
            % The "something" can be "result", "homescore", "awayscore", "scores"
            % For the former, we can chose "home"/"draw"/"away"
            % and for the others, it is an integer constraint. For
            % scores, the constraint is obviously a vector
            [ event, constraint] = process_options(...
                varargin, ...
                'event', 'result', ...
                'constraint', 'home');
            
            count = 0;
            for round = 1: obj.GetNoRounds()
                for team = 1: obj.GetNoTeams()
                    % When looping, only count the games where the current
                    % team plays at home in the current round. This way,
                    % each game counts exactly once
                    if obj.playsAtHome(team, round)
                        hGoals = obj.actualGoalsScored(team, round);
                        aGoals = obj.actualGoalsScored(obj.opposition(team, round), round);
                        switch event
                            case 'result'
                                switch constraint
                                    case 'home'
                                        if hGoals>aGoals, count = count + 1; end
                                    case 'draw'
                                        if hGoals==aGoals, count = count + 1; end
                                    case 'away'
                                        if hGoals<aGoals, count = count + 1; end
                                    otherwise
                                        warning(['Database::CountGames got strange constraint ' constraint] );
                                end
                                
                            case 'homescore'
                                if (hGoals == constraint), count = count + 1; end
                                
                            case 'awayscore'
                                if (aGoals == constraint), count = count + 1; end
                                
                            case 'scores'
                                if (hGoals == constraint(1)) && (aGoals == constraint(2)), count = count + 1; end
                                
                            otherwise
                                warning(['Database::CountGames got strange event ' event] );
                                
                        end
                    end
                    
                end
                
            end
        end
        function result = GetTeam(obj, input)
            % Translates between numeric name (== idx) and text name of
            % teams. Checks if input is numeric, and if so generates text.
            % If input is text (or cell) array (i.e., not numeric), it returns indexes.
            if isnumeric(input)
                result = cell(size(input));
                for lookingAt = 1:length(input)
                    result{lookingAt} = obj.teamNames{input(lookingAt)};
                end
            else
                if ~iscell(input), input = {input}; end % Make sure it is a cell array; potentially with only one entry
                result = zeros(length(input), 1);
                for lookingAt = 1:length(input)
                    result(lookingAt) = strcmp({input{lookingAt}},obj.teamNames)*((1:obj.GetNoTeams())');
                end
            end
        end
        function OK = Verify(obj)
            
            OK = 1;
            for teamIdx = 1: obj.GetNoTeams()
                for roundIdx = 1:obj.GetNoRounds()
                    % Check that for each round <t>, when team <i> plays
                    % team <j>, then team <j> also plays team <i>.
                    % Also check that exactly one has home turf
                    currentOpposition = obj.opposition(teamIdx, roundIdx);
                    if obj.opposition(currentOpposition, roundIdx) ~= teamIdx
                        OK = 0;
                        fprintf('Validation problem: Team %d plays %d, but %d plays %d\n', ...
                            teamIdx, currentOpposition, currentOpposition, obj.opposition(currentOpposition, roundIdx));
                    end
                    if (obj.playsAtHome(teamIdx, roundIdx) + ...
                            obj.playsAtHome(currentOpposition, roundIdx) ~= 1)
                        OK = 0;
                        fprintf('Validation problem: Team %d plays %d. Homefield adv.: %d and %d\n', ...
                            teamIdx, currentOpposition, obj.playsAtHome(teamIdx, roundIdx), ...
                            obj.playsAtHome(currentOpposition, roundIdx));
                    end
                    % Check that the no. shots fired is not less than the
                    % no. goals.
                    if (obj.actualGoalsScored(teamIdx, roundIdx) > ...
                            obj.actualFiredShots(teamIdx, roundIdx))
                        OK = 0;
                        fprintf('Validation problem: Team %d has scored more than the no. shots (Round %d)\n', ...
                            teamIdx, roundIdx);
                    end
                    
                end
            end
            
            % Check that we can translate the data we have in the DB to the
            % gameIdx - representation and back without loosing anything
            [transSchedule, transResults]= obj.TranslateTeamRound2Game();
            [transOpposition, transPlaysAtHome, transGoalsScored, transFiredShots]= ...
                obj.TranslateGame2TeamRound(transSchedule, transResults);
            
            % Check each of the translated results
            if max(max(abs(transOpposition - obj.opposition))) > 0
                disp('Validation problem: opposition did not translate right');
                OK = 0;
            end
            if max(max(abs(transPlaysAtHome - obj.playsAtHome))) > 0
                disp('Validation problem: playsAtHome did not translate right');
                OK = 0;
            end
            if max(max(abs(transGoalsScored - obj.goalsScored))) > 0
                disp('Validation problem: goalsScored did not translate right');
                OK = 0;
            end
            if max(max(abs(transFiredShots - obj.firedShots))) > 0
                disp('Validation problem: firedShots did not translate right');
                OK = 0;
            end
        end
        
        % Hide routines -- to take care of simulations (pretending data to
        % be missing)
        function obj = HideVariable(obj, variable)
            % Hid a variable form the slater coming simulation --> Just
            % NAN-it! Will make the same JAGS files work well, too.
            if ~iscell(variable), variable = {variable}; end
            for idx = 1:length(variable) % runn over cell entries
                operation = ['obj.' char(variable(idx)) ' = nan(size(obj.'  char(variable(idx)) '));'];
                eval(operation);
            end
            warning('So far I do not have a nice initialization of each hidden variable. This MUST be fixed if the hide-feature is to be meaningful/useful.');
        end
        function obj = HideResult(obj, varargin)
            [ teamIdx, teamName, round, hideProbability ] = process_options(...
                varargin, ...
                'teamIdx', 0, ...
                'teamName', {}, ...
                'round', [], ...
                'hideProbability', 0);
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('HideResult both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = obj.GetTeam(teamName);
            end
            
            if (isempty(teamIdx) || any(teamIdx == 0)) && (~isempty(round))
                % Want to blank rounds. If I did not specify which team,
                % then do so for all of them
                teamIdx = 1:obj.GetNoTeams();
            end
            
            if any(teamIdx ~= 0) && (isempty(round))
                % Want to blank som results for a given team, but did not
                % say which rounds. Default is: Do only the last one
                round = obj.GetNoRounds();
            end
            
            % Clear the applicable rounds
            % NOTE!! If team <i> plays team <j> in round t, and team <i>'s
            % game is selected to be blanked, then team <j>'s result is
            % immediately also blanked, obviously.
            for roundIdx = 1:length(round)
                curretOpposition = obj.opposition(teamIdx, round(roundIdx));
                obj.goalsScored(teamIdx, round(roundIdx)) = NaN;
                obj.goalsScored(curretOpposition, round(roundIdx)) = NaN;
                obj.firedShots(teamIdx, round(roundIdx)) = NaN;
                obj.firedShots(curretOpposition, round(roundIdx)) = NaN;
            end
            
            if hideProbability > 0
                % This one generates a binary mask, and hides all the
                % reults with that probability.
                % NOTE! Due to symetric stuff (above), we look at probability/2
                % when choosing each team (as each game can be chosen
                % twice -- once per team in action).
                
                picker = rand(obj.GetNoTeams(), obj.GetNoRounds());
                picker(picker > 1 - hideProbability/2) = 1;
                picker(picker < 1) = 0;
                % Now, if picker(i,t)==1, game involving team i in round t
                % is to be blanked
                for teamIdx = 1:obj.GetNoTeams()
                    for roundIdx = 1:obj.GetNoRounds()
                        if picker(teamIdx, roundIdx)==1
                            curretOpposition = obj.opposition(teamIdx, roundIdx);
                            obj.goalsScored(teamIdx, roundIdx) = NaN;
                            obj.goalsScored(curretOpposition, roundIdx) = NaN;
                            obj.firedShots(teamIdx, roundIdx) = NaN;
                            obj.firedShots(curretOpposition, roundIdx) = NaN;
                        end
                    end
                end
            end
        end
        
        
        %Larger HideResults function for testing with larger datasets. Want
        %to keep both as we might need to run the older methods of maher,
        %dixon or rue again. 
        
        function obj = HideResult_DetailedDataVersion(obj, varargin)
            [ teamIdx, teamName, round, hideProbability ] = process_options(...
                varargin, ...
                'teamIdx', 0, ...
                'teamName', {}, ...
                'round', [], ...
                'hideProbability', 0);
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('HideResult both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = obj.GetTeam(teamName);
            end
            
            if (isempty(teamIdx) || any(teamIdx == 0)) && (~isempty(round))
                % Want to blank rounds. If I did not specify which team,
                % then do so for all of them
                teamIdx = 1:obj.GetNoTeams();
            end
            
            if any(teamIdx ~= 0) && (isempty(round))
                % Want to blank som results for a given team, but did not
                % say which rounds. Default is: Do only the last one
                round = obj.GetNoRounds();
            end
            
            % Clear the applicable rounds
            % NOTE!! If team <i> plays team <j> in round t, and team <i>'s
            % game is selected to be blanked, then team <j>'s result is
            % immediately also blanked, obviously.
            for roundIdx = 1:length(round)
                currentOpposition = obj.opposition(teamIdx, round(roundIdx));
                obj.goalsScored(teamIdx, round(roundIdx)) = NaN;
                obj.goalsScored(currentOpposition, round(roundIdx)) = NaN;
                obj.firedShots(teamIdx, round(roundIdx)) = NaN;
                obj.firedShots(currentOpposition, round(roundIdx)) = NaN;
                obj.shotsOnTarget(teamIdx, round(roundIdx)) = NaN;
                obj.shotsOnTarget(currentOpposition, round(roundIdx)) = NaN;
                obj.shotsBlocked(teamIdx, round(roundIdx)) = NaN;
                obj.shotsBlocked(currentOpposition, round(roundIdx)) = NaN;
                obj.possession(teamIdx, round(roundIdx)) = NaN;
                obj.possession(currentOpposition, round(roundIdx)) = NaN;
                obj.firedPasses(teamIdx, round(roundIdx)) = NaN;
                obj.firedPasses(currentOpposition, round(roundIdx)) = NaN;
                obj.passesOnTarget(teamIdx, round(roundIdx)) = NaN;
                obj.passesOnTarget(currentOpposition, round(roundIdx)) = NaN;
                obj.wonContests(teamIdx, round(roundIdx)) = NaN;
                obj.wonContests(currentOpposition, round(roundIdx)) = NaN;
                obj.tackles(teamIdx, round(roundIdx)) = NaN;
                obj.tackles(currentOpposition, round(roundIdx)) = NaN;
                obj.airials(teamIdx, round(roundIdx)) = NaN;
                obj.airials(currentOpposition, round(roundIdx)) = NaN;
            end
            
            if hideProbability > 0
                % This one generates a binary mask, and hides all the
                % reults with that probability.
                % NOTE! Due to symetric stuff (above), we look at probability/2
                % when choosing each team (as each game can be chosen
                % twice -- once per team in action).
                
                picker = rand(obj.GetNoTeams(), obj.GetNoRounds());
                picker(picker > 1 - hideProbability/2) = 1;
                picker(picker < 1) = 0;
                % Now, if picker(i,t)==1, game involving team i in round t
                % is to be blanked
                for teamIdx = 1:obj.GetNoTeams()
                    for roundIdx = 1:obj.GetNoRounds()
                        if picker(teamIdx, roundIdx)==1
                            curretOpposition = obj.opposition(teamIdx, roundIdx);
                            obj.goalsScored(teamIdx, roundIdx) = NaN;
                            obj.goalsScored(curretOpposition, roundIdx) = NaN;
                            obj.firedShots(teamIdx, roundIdx) = NaN;
                            obj.firedShots(curretOpposition, roundIdx) = NaN;
                        end
                    end
                end
            end
        end
        
        % Get and set routines
        function value= GetNoTeams(obj)
            value = obj.noTeams;
        end
        function value= GetNoRounds(obj)
            value = obj.noRounds;
        end
        function value= GetNoGames(obj)
            value = obj.GetNoTeams() * obj.GetNoRounds() / 2;
        end
        function value = GetGoals(obj, teamIdx, round)
            value = obj.actualGoalsScored(teamIdx, round);
        end
		function value = GetLastRoundWithResults(obj)
            counter = repmat(1:obj.GetNoRounds(),obj.GetNoTeams(),1);
            counter(isnan(obj.goalsScored))=0;
            value = max(max(counter));
        end
        function obj = set.noTeams(obj, value)
            obj.noTeams = value;
            obj = obj.UpdateAfterNoTeamChange(value);
        end
        
        % Translation procedures -- change between (team&round) and
        % (match-id) representations
        function [outSchedule, outResults] = TranslateTeamRound2Game(obj)
            % Create place-holders
            noGames = obj.GetNoTeams() * obj.GetNoRounds()/2;
            outSchedule = zeros(noGames, 3);
            outResults = zeros(noGames, 6);

            % Generate list of all games:
            allGames = GameList(obj, 'teamIdx', 0, 'round', 1:obj.GetNoRounds(), 'verbose', 0);
            
            % Define schedule with three columns: Round, Home, Away
            % and the result with 6 columns: home, away, homeGoal,
            % awayGoal, homeShot, awayShot
            for idx = 1:noGames
                theGame = allGames.GetGame( idx );
                outSchedule(idx, :) = [theGame.round, theGame.homeTeamIdx, theGame.awayTeamIdx];
                outResults(idx, :) = [theGame.homeTeamIdx, theGame.awayTeamIdx, ...
                    obj.goalsScored(theGame.homeTeamIdx, theGame.round),...
                    obj.goalsScored(theGame.awayTeamIdx, theGame.round),...
                    obj.firedShots(theGame.homeTeamIdx, theGame.round),...
                    obj.firedShots(theGame.awayTeamIdx, theGame.round)];
            end            
        end
        
        % Translation procedures -- change between (team&round) and
        % (match-id) representations
        function [outSchedule, outResults] = TranslateTeamRound2Game_DetailedDataVersion(obj)
            % Create place-holders
            noGames = obj.GetNoTeams() * obj.GetNoRounds()/2;
            outSchedule = zeros(noGames, 3);
            outResults = zeros(noGames, 22);

            % Generate list of all games:
            allGames = GameList(obj, 'teamIdx', 0, 'round', 1:obj.GetNoRounds(), 'verbose', 0);
            
            % Define schedule with three columns: Round, Home, Away
            % and the result with 6 columns: home, away, homeGoal,
            % awayGoal, homeShot, awayShot
            for idx = 1:noGames
                theGame = allGames.GetGame( idx );
                outSchedule(idx, :) = [theGame.round, theGame.homeTeamIdx, theGame.awayTeamIdx];
                outResults(idx, :) = [theGame.homeTeamIdx, theGame.awayTeamIdx, ...
                    obj.goalsScored(theGame.homeTeamIdx, theGame.round),...
                    obj.goalsScored(theGame.awayTeamIdx, theGame.round),...
                    obj.firedShots(theGame.homeTeamIdx, theGame.round),...
                    obj.firedShots(theGame.awayTeamIdx, theGame.round),...
                    obj.shotsOnTarget(theGame.homeTeamIdx, theGame.round),...
                    obj.shotsOnTarget(theGame.awayTeamIdx, theGame.round),...
                    obj.shotsBlocked(theGame.homeTeamIdx, theGame.round),...
                    obj.shotsBlocked(theGame.awayTeamIdx, theGame.round),...
                    obj.possession(theGame.homeTeamIdx, theGame.round),...
                    obj.possession(theGame.awayTeamIdx, theGame.round),...
                    obj.firedPasses(theGame.homeTeamIdx, theGame.round),...
                    obj.firedPasses(theGame.awayTeamIdx, theGame.round),...
                    obj.passesOnTarget(theGame.homeTeamIdx, theGame.round),...
                    obj.passesOnTarget(theGame.awayTeamIdx, theGame.round),...
                    obj.wonContests(theGame.homeTeamIdx, theGame.round),...
                    obj.wonContests(theGame.awayTeamIdx, theGame.round),...
                    obj.tackles(theGame.homeTeamIdx, theGame.round),...
                    obj.tackles(theGame.awayTeamIdx, theGame.round),...
                    obj.airials(theGame.homeTeamIdx, theGame.round),...
                    obj.airials(theGame.awayTeamIdx, theGame.round)];
            end            
        end

        
        function [outOpposition, outPlaysAtHome, outGameDate, outGameOrder, outGoalsScored, outFiredShots, ...
                outShotsOnTarget, outShotsBlocked, outPossession, outFiredPasses, outPassesOnTarget, outWonContests, ...
                outTackles, outAirials] ...
                = TranslateGame2TeamRound(obj, inSchedule, inResults)
            % Create place-holders
            outOpposition = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outPlaysAtHome = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outGameDate = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outGameOrder = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outGoalsScored = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outFiredShots = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outShotsOnTarget = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outShotsBlocked = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outPossession = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outFiredPasses = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outPassesOnTarget = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outWonContests = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outTackles = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            outAirials = zeros(obj.GetNoTeams(), obj.GetNoRounds());
            
            % Translate schedule. This matrix has three columns, holding the
            % following information:
            % 1. Round
            % 2. Home - team
            % 3. Away - team
            % 4. Date of the game
            for match = 1:size(inSchedule, 1)
                outOpposition(inSchedule(match,2), inSchedule(match,1)) = inSchedule(match,3);
                outOpposition(inSchedule(match,3), inSchedule(match,1)) = inSchedule(match,2);
                outPlaysAtHome(inSchedule(match,2), inSchedule(match,1)) = 1;
                outGameDate(inSchedule(match,2), inSchedule(match,1)) = inSchedule(match,4);
                outGameDate(inSchedule(match,3), inSchedule(match,1)) = inSchedule(match,4);
            end
            
            % Fix the "game order" stuff. gameOrder(i,t)=j, means that for
            % team i, the t'th game they plsyed was the one scheduled in
            % round j
            for team = 1:obj.GetNoTeams()
                [~, mySortedGames] = sort(outGameDate(team,:));
              outGameOrder(team, :)   = mySortedGames;
            end
            
            
            % And finally recode the results-matrix. The matrix has 6 columns,
            % holding he following information
            % 1. Home - team
            % 2. Away - team
            % 3. HomeTeam goals
            % 4. AwayTeam goals
            % 5. HomeTeam shots
            % 6. AwayTeam shots
            %
            % The reason for not just moving the information as above, when
            % info about who played when, is that we don't know if all games
            % have been played as and when they should --> Must "search"
            % correct index for this to work.
            
            for match = 1:size(inResults, 1)
                homeTeam = inResults(match,1);
                awayTeam = inResults(match,2);
                
                % Look for which game this is
                gameIdx = find(outOpposition(homeTeam,:)==awayTeam);
                % gameIdx has two indexes, as the two teams have played
                % eachother twice.
                % Chose the one where <homeTeam> has played at home...
                if outPlaysAtHome(homeTeam, gameIdx(1))
                    gameIdx = gameIdx(1);
                else
                    gameIdx = gameIdx(2);
                end
                % And we are ready:
                outGoalsScored(homeTeam, gameIdx) = inResults(match, 3);
                outGoalsScored(awayTeam, gameIdx) = inResults(match, 4);
                outFiredShots(homeTeam, gameIdx)  = inResults(match, 5);
                outFiredShots(awayTeam, gameIdx)  = inResults(match, 6);
                %%%%%%%%%%%%%%%% NEW
                
                outShotsOnTarget(homeTeam,gameIdx) = inResults(match, 7);
                outShotsOnTarget(awayTeam,gameIdx) = inResults(match, 8);
                outShotsBlocked(homeTeam,gameIdx) = inResults(match, 9);
                outShotsBlocked(awayTeam,gameIdx) = inResults(match, 10);
                outPossession(homeTeam,gameIdx) = inResults(match,11);
                outPossession(awayTeam,gameIdx) = inResults(match, 12);
                outFiredPasses(homeTeam,gameIdx) = inResults(match, 13);
                outFiredPasses(awayTeam,gameIdx) = inResults(match, 14);
                outPassesOnTarget(homeTeam,gameIdx) = inResults(match, 15);
                outPassesOnTarget(awayTeam,gameIdx) = inResults(match, 16);
                outWonContests(homeTeam,gameIdx) = inResults(match, 17);
                outWonContests(awayTeam,gameIdx) = inResults(match, 18);
                outTackles(homeTeam,gameIdx) = inResults(match, 19);
                outTackles(awayTeam,gameIdx) = inResults(match, 20);
                outAirials(homeTeam,gameIdx) = inResults(match,21);
                outAirials(awayTeam,gameIdx) = inResults(match,22);
                %%%%%%%%%%%%%%%%
            end
        end
        
        function intensity = getIntensity(obj)
            %tacklesHome+tacklesAway+arialsWonHome+arialsWonAway+contestsWonHome+contestsWonAway
            divisors = zeros(obj.GetNoTeams, obj.GetNoRounds);
            intensity = zeros(obj.GetNoTeams, obj.GetNoRounds);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    divisors(i,j) = obj.tackles(i,j)+obj.tackles(obj.opposition(i,j),j)...
                                    +obj.airials(i,j)+obj.airials(obj.opposition(i,j),j)...
                                    +obj.wonContests(i,j)+obj.wonContests(obj.opposition(i,j),j);
                end
            end
            intensityBasic = (obj.tackles+obj.airials+obj.wonContests)./divisors;
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    intensityTeamA= intensityBasic(i,j)*obj.possession(i,j);
                    intensityTeamB = intensityBasic(obj.opposition(i,j),j)*obj.possession(obj.opposition(i,j),j);
                    intensity(i,j) = intensityTeamA/(intensityTeamA+intensityTeamB);
                end
            end 
        end
        
        function efficiency = getEfficiency(obj)
            efficiency = obj.shotsOnTarget ./ obj.firedShots;
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    if obj.firedShots(i,j) == 0
                        efficiency(i,j) = 0;
                    end
                end
            end        
        end
        
        function gameSpeed = getGameSpeed(obj)
            gameSpeed = obj.firedPasses ./ (obj.possession*0.9);
        end
        
        function normalizedEfficiency = getNormEfficiency(obj)
            normalizedEfficiency = zeros(obj.GetNoTeams, obj.GetNoRounds);
            effBasic = obj.shotsOnTarget./obj.firedShots;
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    if obj.firedShots(i,j) == 0
                        effBasic(i,j) = 0;
                    end
                end
            end
            
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    effTeamA= effBasic(i,j);
                    effTeamB = effBasic(obj.opposition(i,j),j);
                    normalizedEfficiency(i,j) = effTeamA/(effTeamA+effTeamB);
                    if effTeamA == 0 && effTeamB == 0
                        normalizedEfficiency(i,j) = 0.5;
                    end
                end
            end 
        end
        
        function shotsMade = getShotsNormalized(obj)
            total = zeros(obj.GetNoTeams, obj.GetNoRounds);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    total(i,j) = obj.shotsOnTarget(i,j)+obj.shotsOnTarget(obj.opposition(i,j),j);
                end
            end
            shotsMade = obj.shotsOnTarget./total;
        end
        
        function shotsMade = getShotsNormalizedLOGIT(obj)
            total = zeros(obj.GetNoTeams, obj.GetNoRounds);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    total(i,j) = obj.shotsOnTarget(i,j)+obj.shotsOnTarget(obj.opposition(i,j),j);
                end
            end
            shotsMade = obj.shotsOnTarget./total;
            shotsMade = -log(1./ shotsMade -1);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    if shotsMade(i,j) == -inf
                        %obj.GetTeam(i)
                        %obj.GetTeam(obj.opposition(i,j))
                        %obj.shotsOnTarget(i,j)
                        %obj.shotsOnTarget(obj.opposition(i,j),j)
                        shotsMade(i,j) =-1000000;
                    elseif shotsMade(i,j) == inf
                        shotsMade(i,j) =1000000;
                    end
                end
            end
        end
        
        function intensity = getIntensityLOGIT(obj)
            intensity = zeros(obj.GetNoTeams, obj.GetNoRounds);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    intensity = obj.getIntensity();
                end
            end
            intensity = -log(1./ intensity -1);
            for i = 1:obj.GetNoTeams
                for j=1:obj.GetNoRounds
                    if intensity(i,j) == -inf
                        intensity(i,j) =-1000000;
                    elseif intensity(i,j) == inf
                        intensity(i,j) =1000000;
                    end
                end
            end
        end
        
        
        
    end
end

