classdef Game
    % GAME holds information about a game, either an observed one or one
    % with sampled results. It is basically only a struct, but with some
    % interfacing.
    
    properties
        homeTeamIdx             % index in the range 1..#teams
        homeTeamName            % text-version of name
        awayTeamIdx             % index in the range 1..#teams
        awayTeamName            % text-version of name
        round                   % index in the range 1..#rounds
        homeGoals               % Positive integer value (or distribution over it)
        awayGoals               % Positive integer value (or distribution over it)
        winner                  % Vector of length 3.
        %                         First element is Prob of home victory, second relates to draws,
        %                         third is prob of away victory. If the result is known, the
        %                         elements will for instance be [0, 1, 0] to signify a draw.
        goalDistributionHome    % Prob of 0, 1, 2, ... goals for Home team. Only valid when used with samples
        goalDistributionAway    % Prob of 0, 1, 2, ... goals for Away team. Only valid when used with samples
        gameSimulated           % If this is a 1 it is a simulated result, otherwise it is an observation
    end
    
    methods
        function obj = Game(homeIdx, homeTxt, awayIdx, awayTxt, round, hGoal, aGoal, distrHome, distrAway)
            obj.homeTeamIdx  = homeIdx;
            obj.homeTeamName = homeTxt;
            obj.awayTeamIdx  = awayIdx;
            obj.awayTeamName = awayTxt;
            obj.round = round;
            obj.homeGoals = hGoal;
            obj.awayGoals = aGoal;
            obj.winner = zeros(3, 1);
            if hGoal > aGoal,  obj.winner(1)=1; end
            if hGoal == aGoal, obj.winner(2)=1; end
            if hGoal < aGoal,  obj.winner(3)=1; end
            obj.gameSimulated = 0;
            if nargin > 7
                obj.goalDistributionHome = distrHome;
                obj.goalDistributionAway = distrAway;
                obj.gameSimulated = 1;
            end
        end
        function OK = PresentGame(obj, varargin)
            
            [verbose] = process_options(...
                varargin, ...
                'verbose',      1);      % Verbose level? 0: Quiet. 1: Minimal: 2: Talkative
            
            fprintf('Round %d: %s - %s:',...
                obj.round, ...
                char(obj.homeTeamName), ...
                char(obj.awayTeamName));
            
            if obj.gameSimulated
                if verbose == 1
                    fprintf('\t\tHome: %2.f%%;   Draw: %2.f%%;   Away: %2.f%%\n', ...
                        100*obj.winner(1), 100*obj.winner(2), 100*obj.winner(3) );
                else
                    fprintf('\n------------------------------------------------------------------\n');
                    fprintf(' - Distribution over outcomes:\n\t\t %2.f%% - %2.f%% - %2.f%%\n', ...
                        100*obj.winner(1), 100*obj.winner(2), 100*obj.winner(3) );
              
                
                    % Find the distribution over results (assuming the home and
                    % away goals are independently distributed)
                    possibleResult = zeros(3, length(obj.goalDistributionHome)*length(obj.goalDistributionAway));
                    idx = 1;
                    for homeIdx = 1:length(obj.goalDistributionHome)
                        for awayIdx = 1:length(obj.goalDistributionAway)
                            possibleResult(1, idx) = homeIdx-1;
                            possibleResult(2, idx) = awayIdx-1;
                            possibleResult(3, idx) = obj.goalDistributionHome(homeIdx)*obj.goalDistributionAway(awayIdx);
                            idx = idx + 1;
                        end
                    end
                    [~, sortIdx] = sort(possibleResult(3,:), 'descend');
                    possibleResult = possibleResult(:, sortIdx);
                    cummulative = 0; idx = 1;
                    fprintf(' - Distribution over results:\n');
                    while (cummulative < .9) && (possibleResult(3, idx) >= 0.05)
                        fprintf('\t  %d - %d (Probability %2.f%%)\n', ...
                            possibleResult(1, idx), possibleResult(2, idx), 100*possibleResult(3, idx) );
                        idx = idx + 1;
                        cummulative = cummulative + possibleResult(3, idx);
                    end
                    fprintf('\n');
                end
            else
                fprintf(' %d - %d\n', obj.homeGoals, obj.awayGoals);
            end
            OK = 1;
        end
        function [binaryDecision, brierScore, geometricMean, ...
                informationLoss, maxLogLikEst, rankProbScore ] = Compare(obj, comparison, varargin)
            % Compare two games, and give a number of different evaluation
            % scores. The relevant information will be in the two
            % 'winner' vectors
            %
            % All the formulas are from Constantinou and Fenton (2012):
            % Solving the problem of inadequate rules for assessing
            % probabilistic football forecast models
            
            [verbose] = process_options(...
                varargin, ...
                'verbose',      0);      % Verbose level? 0: Quiet. 1: Reporting

            % Sanity check: Am I comparing to game-objects describing the
            % same game?
            if ((obj.homeTeamIdx ~= comparison.homeTeamIdx) || ...
                     (obj.awayTeamIdx ~= comparison.awayTeamIdx) || ...
                     (obj.round ~= comparison.round))
                 warning('Game::Compare got two instances representing different games!');
                 return;
            end
                       
            if verbose
                fprintf('Looking at %s vs. %s in round %d. One predictor has distribution (%4.2f, %4.2f, %4.2f), the other (%4.2f, %4.2f, %4.2f)\n', ...
                    char(obj.homeTeamName), char(obj.awayTeamName), obj.round, obj.winner(1), obj.winner(2), obj.winner(3), ...
                    comparison.winner(1), comparison.winner(2), comparison.winner(3));
            end
            
            % binaryDecision
            [~, myGuess] = max(obj.winner); 
            [~, otherGuess] = max(comparison.winner); 
            binaryDecision = (myGuess == otherGuess);
            if verbose, fprintf('  * binaryDecision     \t= %7d\n', binaryDecision); end
            
            % brierScore   --- FIXED
            % Fixing so that bigger number is better, and max value is 1.
            brierScore = sum(  (obj.winner - comparison.winner) .^2  );
            brierScore = 1 - brierScore / 2;
            if verbose, fprintf('  * brierScoreFIX      \t= %7.4f\n', brierScore); end
            
            % geometricMean
            geometricMean = sum(  obj.winner  .* comparison.winner ); 
            if verbose, fprintf('  * geometricMean      \t= %7.4f\n', geometricMean); end
            
            % informationLossFIX
            % Fixing so that it increases with goodness
            informationLoss = log2(geometricMean);
            if verbose, fprintf('  * informationLossFIX \t= %7.4f\n', informationLoss); end
            
            % maxLogLikEst
            maxLogLikEst = log(geometricMean);
            if verbose, fprintf('  * maxLogLikEst       \t= %7.4f\n', maxLogLikEst); end
            
            % rankProbScoreFIX  -- Fixed so that big value is good value
            rankProbScore = 1 - sum((cumsum(obj.winner) - cumsum(comparison.winner)).^2 )/2;
            if verbose, fprintf('  * rankProbScoreFIX\t= %7.4f\n', rankProbScore); end
            
        end
    end
    
    
end

