classdef GameList
    %GAMELIST is basically onli a list of games. Each game is an instance
    %of the Game class. The important stuff here is basically
    
    properties
        list  % A list of games
    end
    
    methods
        function obj = GameList(db, varargin)
            additionalInput = varargin;
            if length(additionalInput)==1, additionalInput = additionalInput{1}; end
            [ teamIdx, teamName, rounds, verbose] = process_options(...
                additionalInput, ...
                'teamIdx', 0, ...
                'teamName', {}, ...
                'round', [], ...
                'verbose', 0);
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Gamelist creator both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = 0; end;
            end
            
            if any(teamIdx == 0) && (~isempty(rounds))
                % Want full rounds. As no team is specified, we look at all
                % games
                teamIdx = 1:db.GetNoTeams();
            end
            
            if any(teamIdx ~= 0) && (isempty(rounds))
                % Want to blank som results for a given team, but did not
                % say which rounds. Default is: Do only the last one
                rounds = db.GetNoRounds();
            end
            
            % For each game, we an instance of the Game class
            
            obj.list = cell(prod([length(teamIdx) length(rounds)]), 1);
            resultIdx = 0;
            for myTeamCounter = 1:length(teamIdx)
                for myRoundCounter = 1:length(rounds)
                    if db.playsAtHome(teamIdx(myTeamCounter), rounds(myRoundCounter))
                        homeIdx = teamIdx(myTeamCounter);
                        awayIdx = db.opposition(teamIdx(myTeamCounter), rounds(myRoundCounter));
                    else
                        homeIdx = db.opposition(teamIdx(myTeamCounter), rounds(myRoundCounter));
                        awayIdx = teamIdx(myTeamCounter);
                    end
                    
                    % Check if this specific game has already been
                    % reported. This can happen when teamA plays teamB, and
                    % I have asked for both teamA and teamB's results
                    foundIt = 0;
                    for checkIdx = 1:resultIdx
                        if ((obj.list{checkIdx}.homeTeamIdx == homeIdx) && ...
                                (obj.list{checkIdx}.awayTeamIdx == awayIdx) && ...
                                (obj.list{checkIdx}.round == rounds(myRoundCounter)))
                            foundIt = 1;
                        end
                    end
                    if foundIt
                        % This is a duplicate. Delete last entry in the
                        % results thingy, then move on to the next game
                        obj.list(end)=[];
                    else
                        resultIdx = resultIdx +1;
                        obj.list{resultIdx} = Game(homeIdx, ...
                            db.GetTeam(homeIdx), ...
                            awayIdx, ...
                            db.GetTeam(awayIdx), ...
                            rounds(myRoundCounter), ...
                            db.GetGoals(homeIdx, rounds(myRoundCounter)), ...
                            db.GetGoals(awayIdx, rounds(myRoundCounter)));
                        obj.list{resultIdx}.gameSimulated = 0;
                        if verbose
                            obj.list{resultIdx}.PresentGame();
                        end
                    end
                end
            end
            obj.list = obj.SortList();
        end
        function obj = Sort(obj)
            obj.list = obj.SortList();
        end
        function myGame = GetGame(obj, idx)
            if nargin < 2
                warning('GameList::GetGame without giving index!');
                idx = 1;
            end
            if idx > length(obj.list)
                warning('GameList::GetGame with too large an index!');
                idx = 1;
            end
            myGame = obj.list{idx};
        end
        function myLength = GetNoGames(obj)
            myLength  = length(obj.list);
        end
        function OK = PresentGames(obj, varargin)
            [verbose] = process_options(...
                varargin, ...
                'verbose',      1);      % Verbose level? 0: Quiet. 1: Minimal: 2: Talkative
            OK = 1;
            for idx = 1:length(obj.list)
                OK = OK & obj.GetGame(idx).PresentGame('verbose', verbose);
            end
        end
        function [binaryDecision, brierScore, geometricMean, informationLoss, maxLogLikEst, rankProbScore ] = Compare(obj, comparison, varargin)
            [verbose] = process_options(...
                varargin, ...
                'verbose',      1);      % Verbose level? 0: Quiet. 1: Gross total numbers; 2: Screen dump of everything
            binaryDecision = 0;
            brierScore = 0;
            geometricMean = 0;
            informationLoss = 0;
            maxLogLikEst = 0;
            rankProbScore = 0;
            
            for idx = 1:length(obj.list)
                
                [binaryDecisionAdd, brierScoreAdd, geometricMeanAdd, ...
                    informationLossAdd, maxLogLikEstAdd, rankProbScoreAdd ] ...
                    = obj.GetGame(idx).Compare(comparison.GetGame(idx), 'verbose', max(0, verbose-1));
                
                binaryDecision = binaryDecision + binaryDecisionAdd;
                brierScore = brierScore + brierScoreAdd;
                geometricMean = geometricMean + geometricMeanAdd;
                informationLoss = informationLoss + informationLossAdd;
                maxLogLikEst = maxLogLikEst + maxLogLikEstAdd;
                rankProbScore = rankProbScore + rankProbScoreAdd;
            end
            noGames = length(obj.list);
            binaryDecision = binaryDecision/noGames;
            brierScore = brierScore/noGames;
            geometricMean = geometricMean/noGames;
            informationLoss = informationLoss/noGames;
            maxLogLikEst = maxLogLikEst/noGames;
            rankProbScore = rankProbScore/noGames;
            
            if verbose
                fprintf('Accuracy results:\n');
                fprintf('  * binaryDecision     \t= %7.4f\n', binaryDecision);                
                fprintf('  * brierScoreFIX      \t= %7.4f\n', brierScore);
                fprintf('  * geometricMean      \t= %7.4f\n', geometricMean);
                fprintf('  * informationLossFIX \t= %7.4f\n', informationLoss);
                fprintf('  * maxLogLikEst       \t= %7.4f\n', maxLogLikEst);
                fprintf('  * rankProbScoreFIX   \t= %7.4f\n', rankProbScore);
            end
            
        end
    end
    
    methods(Access = private)
        function sortedList = SortList(obj)
            % Create a gamelist sorted by
            %  -- round
            %  -- homeTeam index 
            %  -- awayTeam index  ( superflous, as a team can only play
            %  once around (and also only once at home per round)
            
            sortArray = zeros(obj.GetNoGames(), 3);
            for theGameIdx = 1:obj.GetNoGames()
                sortArray(theGameIdx, 1) = obj.GetGame(theGameIdx).round;
                sortArray(theGameIdx, 2) = obj.GetGame(theGameIdx).homeTeamIdx;
                sortArray(theGameIdx, 3) = obj.GetGame(theGameIdx).awayTeamIdx;
            end
            
            [~, newOrder] = sortrows( sortArray );
            sortedList = obj.list(newOrder);
        end
    end
    
end

